from enum import Enum
import random


###################################
#           Rank
###################################

class Rank(Enum):
    ACE = 1
    TWO = 2
    THREE = 3
    FOUR = 4
    FIVE = 5
    SIX = 6
    SEVEN = 7
    EIGHT = 8
    NINE = 9
    TEN = 10
    JACK = 10
    QUEEN = 10
    KING = 10
    

###################################
#           Suit
###################################  
    
class Suit(Enum):
    SPADES = 1
    HEARTS = 2
    CLUBS = 3
    DIAMONDS = 4

###################################
#           Card
###################################
    
class Card():
    rank = None
    suit = None
    
    def __init__(self, rank, suit):
        self.rank = rank
        self.suit = suit
        
    def getValue(self):
        return self.rank.value
    
    def __repr__(self):
        return "{} of {}".format(self.rank.name, self.suit.name)
    
    def __str__(self):
        return "{} of {}".format(self.rank.name, self.suit.name)

###################################
#           Player
###################################
        
class Player():
    name = ""
    hand = []
    
    def __init__(self, name):
        self.name = name
        self.hand = []
        
    def addCard(self, card):
        self.hand.append(card)
        
    def getHand(self):
        return self.hand
    
    def getHandValue(self):
        value = 0
        numAces = 0
        
        for card in self.hand:
            value += card.getValue()
            if(card.getValue() == 1):
                numAces += 1
        
        while (value <= 11 and numAces > 0):
            value += 10
            numAces -= 1
        
        return value
    
    def hasBlackjack(self):
        return len(self.hand) == 2 and self.getHandValue() == 21
    
    def isBust(self):
        return self.getHandValue() > 21
       
###################################
#           Deck
###################################
        
class Deck():
    cards = []
    currentCardIndex = 0
    
    def __init__(self, cards):
        self.cards = cards
        self.currentCardIndex = 0
            
    def shuffle(self):
        
        for i in range(len(self.cards) - 1, 0, -1):
            j = random.randint(0, i)
            temp = self.cards[j]
            self.cards[j] = self.cards[i]
            self.cards[i] = temp
        
        self.currentCardIndex = 0
    
    def dealCard(self):
        
        if(self.currentCardIndex >= len(self.cards)):
            return None
        
        card = self.cards[self.currentCardIndex]
        self.currentCardIndex += 1
        
        return card


###################################
#           Main
###################################

def getPlayerChoice():
    choice = input("Choose an action: (H)it or (S)tand: ")
    return choice      

def createDeck():
    
    cards = []
    
    for suit in range(1, 4):
        for rank in range(1, 10):
            card = createCard(Rank(rank), Suit(suit))
            cards.append(card)
    
    for suit in range(1, 4):
        card = createCard(Rank['JACK'], Suit(suit))
        cards.append(card)
        card = createCard(Rank['QUEEN'], Suit(suit))
        cards.append(card)
        card = createCard(Rank['KING'], Suit(suit))
        cards.append(card)
        
        
    return Deck(cards) 

def createCard(rank, suit):
    return Card(rank, suit)  

def createPlayer(name):
    return Player(name)  

def main():
    
    print('Welcome to Blackjack!')
    
    deck = createDeck()
    deck.shuffle()
    
    dealer = createPlayer("Dealer")
    player = createPlayer("Player")
    
    dealer.addCard(deck.dealCard())
    player.addCard(deck.dealCard())
    dealer.addCard(deck.dealCard())
    player.addCard(deck.dealCard())
    
    gameOver = False
    
    while (not gameOver):
        
        print()
        print("Dealer's Hand: ", dealer.getHand())
        print("Player's Hand: ", player.getHand())
        
        if (dealer.hasBlackjack() and player.hasBlackjack()):
            print("Both dealer and player have Blackjack. It's a tie!")
            gameOver = True
        elif (dealer.hasBlackjack()):
            print("Dealer has Blackjack. Dealer wins.")
            gameOver = True
        elif (player.hasBlackjack()):
            print("Player has Blackjack. Player wins.")
            gameOver = True
        else:
            choice = getPlayerChoice()
            
            if ('h' in choice.lower()):
                player.addCard(deck.dealCard())
                
                if (player.isBust()):
                    print("Player's Hand: ", player.getHand())
                    print("Player busts. Dealer wins.")
                gameOver = True
            elif ('s' in choice.lower()):
                
                while (dealer.getHandValue() < 17):
                    dealer.addCard(deck.dealCard())
                
                print("Dealer's Hand: ", dealer.getHand())
                
                dealerValue = dealer.getHandValue()
                playerValue = player.getHandValue()
                
                if (dealerValue > 21):
                    print("Dealer busts. Player wins.")
                elif (dealerValue > playerValue):
                    print("Dealer wins.")
                elif (dealerValue < playerValue):
                    print("Player wins.")
                else:
                    print("It's a tie.")
                gameOver = True
            else:
                print("Invalid choice. Please try again.")
                
    print("Thank you for playing Blackjack!")
        
    
if __name__ == '__main__':
    main()
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    